/***************************************************************************
 *   Copyright (C) 2010~2012 by CSSlayer                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.              *
 ***************************************************************************/

#include <limits.h>

#include "fcitx/fcitx.h"
#include "fcitx/frontend.h"
#include "fcitx-utils/utils.h"
#include "fcitx/instance.h"
#include "fcitx/module.h"
#include "fcitx-utils/log.h"
#include "fcitx/configfile.h"
#include "fcitx/hook.h"
#include "module/remote/remote.h"
#include "cfe.h"

static void* CFECreate(FcitxInstance* instance, int frontendid);
static boolean CFEDestroy(void* arg);
void CFECreateIC(void* arg, FcitxInputContext* context, void *priv);
boolean CFECheckIC(void* arg, FcitxInputContext* context, void* priv);
void CFEDestroyIC(void* arg, FcitxInputContext* context);
static void CFEEnableIM(void* arg, FcitxInputContext* ic);
static void CFECloseIM(void* arg, FcitxInputContext* ic);
static void CFECommitString(void* arg, FcitxInputContext* ic, const char* str);
static void CFEForwardKey(void* arg, FcitxInputContext* ic, FcitxKeyEventType event, FcitxKeySym sym, unsigned int state);
static void CFEUpdatePreedit(void* arg, FcitxInputContext* ic);
static boolean CFECheckICFromSameApplication(void* arg, FcitxInputContext* icToCheck, FcitxInputContext* ic);
static void CFEUpdateIMList(void* arg);
static void CFEUpdateCurrentIM(void* arg);
static void CFEUpdateIMInfoForIC(void* arg);

FCITX_DEFINE_PLUGIN(fcitx_cfe, frontend, FcitxFrontend) = {
    CFECreate,
    CFEDestroy,
    CFECreateIC,
    CFECheckIC,
    CFEDestroyIC,
    CFEEnableIM,
    CFECloseIM,
    CFECommitString,
    CFEForwardKey,
    NULL,
    NULL,
    CFEUpdatePreedit,
    NULL,
    NULL,
    CFECheckICFromSameApplication,
    NULL,
    NULL,
    NULL
};

void* CFECreate(FcitxInstance* instance, int frontendid)
{
    FcitxCFEFrontend* cfe = fcitx_utils_malloc0(sizeof(FcitxCFEFrontend));
    cfe->frontendid = frontendid;
    cfe->owner = instance;

    FcitxIMEventHook hook;
    hook.arg = cfe;
    hook.func = CFEUpdateIMList;
    FcitxInstanceRegisterUpdateIMListHook(instance, hook);
    hook.func = CFEUpdateCurrentIM;
    FcitxInstanceRegisterIMChangedHook(instance, hook);
    hook.func = CFEUpdateIMInfoForIC;
    FcitxInstanceRegisterInputFocusHook(instance, hook);

    return cfe;
}

boolean CFEDestroy(void* arg)
{
    return true;
}

void CFECreateIC(void* arg, FcitxInputContext* context, void* priv)
{
}

boolean CFECheckIC(void* arg, FcitxInputContext* context, void* priv)
{
    return true;
}

void CFEDestroyIC(void* arg, FcitxInputContext* context)
{
}

void CFEEnableIM(void* arg, FcitxInputContext* ic)
{
}

void CFECloseIM(void* arg, FcitxInputContext* ic)
{
}

void CFECommitString(void* arg, FcitxInputContext* ic, const char* str)
{
    RemoteCommitString(str);
}

void CFEForwardKey(void* arg, FcitxInputContext* ic, FcitxKeyEventType event, FcitxKeySym sym, unsigned int state)
{
}

void CFEUpdatePreedit(void* arg, FcitxInputContext* ic)
{
}

boolean CFECheckICFromSameApplication(void* arg, FcitxInputContext* icToCheck, FcitxInputContext* ic)
{
    return true;
}

void CFEUpdateIMList(void* arg)
{
}

void CFEUpdateCurrentIM(void* arg)
{
}

void CFEUpdateIMInfoForIC(void* arg)
{
}

// kate: indent-mode cstyle; space-indent on; indent-width 0;
